@@ -4,7 +4,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2018	PgPool Global Development Group
+ * Copyright (c) 2003-2019	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -133,6 +133,9 @@ pool_init_session_context(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * bac
 	/* Backends have not ignored messages yet */
 	pool_unset_ignore_till_sync();
 
+	/* Unset suspend reading from frontend flag */
+	pool_unset_suspend_reading_from_frontend();
+
 	/* Initialize where to send map for PREPARE statements */
 #ifdef NOT_USED
 	memset(&session_context->prep_where, 0, sizeof(session_context->prep_where));
@@ -481,6 +484,30 @@ pool_clear_sent_message_list(void)
 	}
 }
 
+/*
+ * Zap query context info in sent messages to indicate that the query context
+ * has been already removed.
+ */
+void
+pool_zap_query_context_in_sent_messages(POOL_QUERY_CONTEXT *query_context)
+{
+	int			i;
+	POOL_SENT_MESSAGE_LIST *msglist;
+
+	msglist = &pool_get_session_context(false)->message_list;
+
+	for (i = 0; i < msglist->size; i++)
+	{
+		elog(LOG, "checking zapping sent message: %p query_context: %p",
+			 &msglist->sent_messages[i], msglist->sent_messages[i]->query_context);
+		if (msglist->sent_messages[i]->query_context == query_context)
+		{
+			msglist->sent_messages[i]->query_context = NULL;
+			elog(LOG, "Zap sent message: %p", &msglist->sent_messages[i]);
+		}
+	}
+}
+
 static void
 dump_sent_message(char *caller, POOL_SENT_MESSAGE * m)
 {
@@ -594,7 +621,7 @@ pool_add_sent_message(POOL_SENT_MESSAGE * message)
 }
 
 /*
- * Get a sent message
+ * Find a sent message by kind and name.
  */
 POOL_SENT_MESSAGE *
 pool_get_sent_message(char kind, const char *name, POOL_SENT_MESSAGE_STATE state)
@@ -618,6 +645,29 @@ pool_get_sent_message(char kind, const char *name, POOL_SENT_MESSAGE_STATE state
 	return NULL;
 }
 
+/*
+ * Find a sent message by query context.
+ */
+POOL_SENT_MESSAGE *
+pool_get_sent_message_by_query_context(POOL_QUERY_CONTEXT * query_context)
+{
+	int			i;
+	POOL_SENT_MESSAGE_LIST *msglist;
+
+	msglist = &pool_get_session_context(false)->message_list;
+
+	if (query_context == NULL)
+		return NULL;
+
+	for (i = 0; i < msglist->size; i++)
+	{
+		if (msglist->sent_messages[i]->query_context == query_context)
+			return msglist->sent_messages[i];
+	}
+
+	return NULL;
+}
+
 /*
  * Set message state to POOL_SENT_MESSAGE_STATE to POOL_SENT_MESSAGE_CLOSED.
  */
@@ -1709,6 +1759,33 @@ pool_get_minor_version(void)
 	return 0;
 }
 
+/*
+ * Is suspend_reading_from_frontend flag set?
+ */
+bool
+pool_is_suspend_reading_from_frontend(void)
+{
+	return session_context->suspend_reading_from_frontend;
+}
+
+/*
+ * Set suspend_reading_from_frontend flag.
+ */
+void
+pool_set_suspend_reading_from_frontend(void)
+{
+	session_context->suspend_reading_from_frontend = true;
+}
+
+/*
+ * Unset suspend_reading_from_frontend flag.
+ */
+void
+pool_unset_suspend_reading_from_frontend(void)
+{
+	session_context->suspend_reading_from_frontend = false;
+}
+
 #ifdef NOT_USED
 /*
  * Set preferred "master" node id.
