@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2015	PgPool Global Development Group
+ * Copyright (c) 2003-2019	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -851,6 +851,7 @@ static POOL_CONNECTION_POOL * new_connection(POOL_CONNECTION_POOL * p)
 	POOL_CONNECTION_POOL_SLOT *s;
 	int			active_backend_count = 0;
 	int			i;
+	bool		status_changed = false;
 
 	MemoryContext oldContext = MemoryContextSwitchTo(TopMemoryContext);
 
@@ -882,6 +883,9 @@ static POOL_CONNECTION_POOL * new_connection(POOL_CONNECTION_POOL * p)
 					(errmsg("creating new connection to backend"),
 					 errdetail("skipping backend slot %d because global backend_status = %d",
 							   i, BACKEND_INFO(i).backend_status)));
+
+			/* sync local status with global status */
+			*(my_backend_status[i]) = BACKEND_INFO(i).backend_status;
 			continue;
 		}
 
@@ -915,7 +919,7 @@ static POOL_CONNECTION_POOL * new_connection(POOL_CONNECTION_POOL * p)
 					/* set down status to local status area */
 					*(my_backend_status[i]) = CON_DOWN;
 
-					/* if master_node_id is not updated, the update it */
+					/* if master_node_id is not updated, then update it */
 					if (Req_info->master_node_id == i)
 					{
 						int			old_master = Req_info->master_node_id;
@@ -953,11 +957,13 @@ static POOL_CONNECTION_POOL * new_connection(POOL_CONNECTION_POOL * p)
 		{
 			BACKEND_INFO(i).backend_status = CON_UP;
 			pool_set_backend_status_changed_time(i);
+			status_changed = true;
 		}
 		active_backend_count++;
 	}
 
-	(void) write_status_file();
+	if (status_changed)
+		(void) write_status_file();
 
 	MemoryContextSwitchTo(oldContext);
 
